@@ -5,14 +5,6 @@
 
     SPDX-License-Identifier: LGPL-2.0-only
 
-    RFC 1321 "MD5 Message-Digest Algorithm" Copyright (C) 1991-1992.             // krazy:exclude=copyright
-    RSA Data Security, Inc. Created 1991. All rights reserved.
-
-    The KMD5 class is based on a C++ implementation of
-    "RSA Data Security, Inc. MD5 Message-Digest Algorithm" by
-    Mordechai T. Abzug,  Copyright (c) 1995.  This implementation                // krazy:exclude=copyright
-    passes the test-suite as defined in RFC 1321.
-
     The encoding and decoding utilities in KCodecs with the exception of
     quoted-printable are based on the java implementation in HTTPClient
     package by Ronald Tschalär Copyright (C) 1996-1999.                          // krazy:exclude=copyright
@@ -27,12 +19,10 @@
 #include "kcodecs_debug.h"
 #include "kcodecs_p.h"
 #include "kcodecsbase64.h"
-#include "kcodecsidentity.h"
 #include "kcodecsqp.h"
 #include "kcodecsuuencode.h"
 
-#include <QMutex>
-
+#include <array>
 #include <cassert>
 #include <cstring>
 #include <stdio.h>
@@ -40,7 +30,6 @@
 #include <string.h>
 
 #include <QDebug>
-#include <QHash>
 #include <QStringDecoder>
 #include <QStringEncoder>
 
@@ -93,58 +82,58 @@ QByteArray updateEncodingCharset(const QByteArray &currentCharset, const QByteAr
 
 /******************************** KCodecs ********************************/
 
-QByteArray KCodecs::quotedPrintableEncode(const QByteArray &in, bool useCRLF)
+QByteArray KCodecs::quotedPrintableEncode(QByteArrayView in, bool useCRLF)
 {
     Codec *codec = Codec::codecForName("quoted-printable");
     return codec->encode(in, useCRLF ? Codec::NewlineCRLF : Codec::NewlineLF);
 }
 
-void KCodecs::quotedPrintableEncode(const QByteArray &in, QByteArray &out, bool useCRLF)
+void KCodecs::quotedPrintableEncode(QByteArrayView in, QByteArray &out, bool useCRLF)
 {
     out = quotedPrintableEncode(in, useCRLF ? Codec::NewlineCRLF : Codec::NewlineLF);
 }
 
-QByteArray KCodecs::quotedPrintableDecode(const QByteArray &in)
+QByteArray KCodecs::quotedPrintableDecode(QByteArrayView in)
 {
     Codec *codec = Codec::codecForName("quoted-printable");
     return codec->decode(in);
 }
 
-void KCodecs::quotedPrintableDecode(const QByteArray &in, QByteArray &out)
+void KCodecs::quotedPrintableDecode(QByteArrayView in, QByteArray &out)
 {
     out = quotedPrintableDecode(in);
 }
 
-QByteArray KCodecs::base64Encode(const QByteArray &in)
+QByteArray KCodecs::base64Encode(QByteArrayView in)
 {
     Codec *codec = Codec::codecForName("base64");
     return codec->encode(in);
 }
 
-void KCodecs::base64Encode(const QByteArray &in, QByteArray &out, bool insertLFs)
+void KCodecs::base64Encode(QByteArrayView in, QByteArray &out, bool insertLFs)
 {
     Q_UNUSED(insertLFs);
     out = base64Encode(in);
 }
 
-QByteArray KCodecs::base64Decode(const QByteArray &in)
+QByteArray KCodecs::base64Decode(QByteArrayView in)
 {
     Codec *codec = Codec::codecForName("base64");
     return codec->decode(in);
 }
 
-void KCodecs::base64Decode(const QByteArray &in, QByteArray &out)
+void KCodecs::base64Decode(const QByteArrayView in, QByteArray &out)
 {
     out = base64Decode(in);
 }
 
-QByteArray KCodecs::uudecode(const QByteArray &in)
+QByteArray KCodecs::uudecode(QByteArrayView in)
 {
     Codec *codec = Codec::codecForName("x-uuencode");
     return codec->decode(in);
 }
 
-void KCodecs::uudecode(const QByteArray &in, QByteArray &out)
+void KCodecs::uudecode(QByteArrayView in, QByteArray &out)
 {
     out = uudecode(in);
 }
@@ -348,13 +337,13 @@ bool parseEncodedWord(const char *&scursor,
 
 //@endcond
 
-QString KCodecs::decodeRFC2047String(const QString &msg)
+QString KCodecs::decodeRFC2047String(QStringView msg)
 {
     QByteArray usedCS;
     return decodeRFC2047String(msg.toUtf8(), &usedCS, CodecNames::utf8(), NoOption);
 }
 
-QString KCodecs::decodeRFC2047String(const QByteArray &src, QByteArray *usedCS, const QByteArray &defaultCS, CharsetOption charsetOption)
+QString KCodecs::decodeRFC2047String(QByteArrayView src, QByteArray *usedCS, const QByteArray &defaultCS, CharsetOption charsetOption)
 {
     QByteArray result;
     QByteArray spaceBuffer;
@@ -415,7 +404,7 @@ QString KCodecs::decodeRFC2047String(const QByteArray &src, QByteArray *usedCS,
     }
 }
 
-QByteArray KCodecs::encodeRFC2047String(const QString &src, const QByteArray &charset)
+QByteArray KCodecs::encodeRFC2047String(QStringView src, const QByteArray &charset)
 {
     QByteArray result;
     int start = 0;
@@ -527,58 +516,29 @@ QByteArray KCodecs::encodeRFC2047String(const QString &src, const QByteArray &ch
 /******************************************************************************/
 /*                           KCodecs::Codec                                   */
 
-// global list of KCodecs::Codec's
-//@cond PRIVATE
-namespace
+KCodecs::Codec *KCodecs::Codec::codecForName(QByteArrayView name)
 {
-static QHash<QByteArray, KCodecs::Codec *> *allCodecs = nullptr;
-Q_GLOBAL_STATIC(QMutex, dictLock)
+    struct CodecEntry {
+        const char *name;
+        std::unique_ptr<KCodecs::Codec> codec;
+    };
+    // ### has to be sorted by name!
+    static const std::array<CodecEntry, 6> s_codecs{{
+        {"b", std::make_unique<KCodecs::Rfc2047BEncodingCodec>()},
+        {"base64", std::make_unique<KCodecs::Base64Codec>()},
+        {"q", std::make_unique<KCodecs::Rfc2047QEncodingCodec>()},
+        {"quoted-printable", std::make_unique<KCodecs::QuotedPrintableCodec>()},
+        {"x-kmime-rfc2231", std::make_unique<KCodecs::Rfc2231EncodingCodec>()},
+        {"x-uuencode", std::make_unique<KCodecs::UUCodec>()},
+    }};
 
-static void createCodecs()
-{
-    // all->insert( "7bit", new KCodecs::SevenBitCodec() );
-    // all->insert( "8bit", new KCodecs::EightBitCodec() );
-    allCodecs->insert("base64", new KCodecs::Base64Codec());
-    allCodecs->insert("quoted-printable", new KCodecs::QuotedPrintableCodec());
-    allCodecs->insert("b", new KCodecs::Rfc2047BEncodingCodec());
-    allCodecs->insert("q", new KCodecs::Rfc2047QEncodingCodec());
-    allCodecs->insert("x-kmime-rfc2231", new KCodecs::Rfc2231EncodingCodec());
-    allCodecs->insert("x-uuencode", new KCodecs::UUCodec());
-    // all->insert( "binary", new KCodecs::BinaryCodec() );
-}
-
-static void cleanupCodecs()
-{
-    qDeleteAll(*allCodecs);
-    delete allCodecs;
-    allCodecs = nullptr;
-}
-
-}
-//@endcond
-
-KCodecs::Codec *KCodecs::Codec::codecForName(const char *name)
-{
-    const QByteArray ba(name);
-    return codecForName(ba);
-}
-
-KCodecs::Codec *KCodecs::Codec::codecForName(const QByteArray &name)
-{
-    QMutexLocker locker(dictLock()); // protect "allCodecs"
-    if (!allCodecs) {
-        allCodecs = new QHash<QByteArray, Codec *>();
-        qAddPostRoutine(cleanupCodecs);
-        createCodecs();
-    }
-    QByteArray lowerName = name.toLower();
-    Codec *codec = (*allCodecs).value(lowerName);
-
-    if (!codec) {
+    const auto it = std::lower_bound(s_codecs.begin(), s_codecs.end(), name, [](const auto &lhs, auto rhs) {
+        return rhs.compare(lhs.name, Qt::CaseInsensitive) > 0;
+    });
+    if (it == s_codecs.end() || name.compare((*it).name, Qt::CaseInsensitive) != 0) {
         qWarning() << "Unknown codec \"" << name << "\" requested!";
     }
-
-    return codec;
+    return (*it).codec.get();
 }
 
 bool KCodecs::Codec::encode(const char *&scursor, const char *const send, char *&dcursor, const char *const dend, NewlineType newline) const
@@ -607,7 +567,7 @@ bool KCodecs::Codec::encode(const char *&scursor, const char *const send, char *
     return true; // successfully encoded.
 }
 
-QByteArray KCodecs::Codec::encode(const QByteArray &src, NewlineType newline) const
+QByteArray KCodecs::Codec::encode(QByteArrayView src, NewlineType newline) const
 {
     // allocate buffer for the worst case:
     QByteArray result;
@@ -630,7 +590,7 @@ QByteArray KCodecs::Codec::encode(const QByteArray &src, NewlineType newline) co
     return result;
 }
 
-QByteArray KCodecs::Codec::decode(const QByteArray &src, NewlineType newline) const
+QByteArray KCodecs::Codec::decode(QByteArrayView src, NewlineType newline) const
 {
     // allocate buffer for the worst case:
     QByteArray result;
